-- --------------------------------------
-- NeedToKnow
-- by Nephthys <Drunken Monkeys> of Hyjal
-- zomg rawr
-- Inspired by DiscoDice
-- --------------------------------------


-- -------------
-- ADDON GLOBALS
-- -------------

NEEDTOKNOW_VERSION = "1.1";

local NEEDTOKNOW_UPDATE_INTERVAL_BAR = 0.05;
local NEEDTOKNOW_UPDATE_INTERVAL_ICON = 0.3;

local NEEDTOKNOW_DEFAULT_FRAME_WIDTH = 200;
local NEEDTOKNOW_DEFAULT_BAR_WIDTH = 195;
local NEEDTOKNOW_DEFAULT_BAR_BORDER_WIDTH = 256;

local NEEDTOKNOW_BAR_TEXTURES = {
	"BantoBar",
	"Charcoal",
	"Default",
	"Glaze",
	"Otravi",
	"Perl",
	"Smooth",
	"Steel",
	"Striped",
}

function NeedToKnow_Test(stuff)
	if ( stuff ) then
		DEFAULT_CHAT_FRAME:AddMessage("NeedToKnow test: "..stuff);
	else
		DEFAULT_CHAT_FRAME:AddMessage("NeedToKnow test: "..this:GetName());
	end
end



-- ----------------
-- DEFAULT SETTINGS
-- ----------------

local NeedToKnow_Defaults = {
	["BarColor"] = 	{	{ r = 0.8, g = 0.6, b = 0.0 },
						{ r = 0.8, g = 0.2, b = 0.1 },
						{ r = 0.2, g = 0.7, b = 0.1 },
						{ r = 0.2, g = 0.1, b = 0.7 },
					}, 
	["BarEnabled"] = { 1, 1, 1, 1 }, 
	["BarTexture"] = "Default", 
	["BuffWatched"] = { "Slice and Dice", "Blade Flurry", "Find Weakness", "Ghostly Strike" }, 
	["IconsOn"] = 1, 
	["IconsOnlyInCombat"] = nil, 
	["AbilityWatched"] = { "Feint", "Cold Blood", "Adrenaline Rush", "Blade Flurry", "Preparation", "Bladefist's Breadth", "Cloak of Shadows" }, 
	["Locked"] = nil,
	["Scale"] = 1.0,
	["Version"] = NEEDTOKNOW_VERSION,
	["Width"] = 200,
};
-- I use 1/nil instead of true/false for easier integration with Blizzard's UIDropDownMenu code.  



-- ----------
-- MAIN FRAME
-- ----------

function NeedToKnow_Frame_OnLoad()
	this:RegisterEvent("PLAYER_LOGIN");
	this:RegisterForDrag("LeftButton");
	SlashCmdList["NEEDTOKNOW"] = NeedToKnow_SlashCommand;
	SLASH_NEEDTOKNOW1 = "/needtoknow";
	SLASH_NEEDTOKNOW2 = "/ntk";
--	DEFAULT_CHAT_FRAME:AddMessage(format(NEEDTOKNOW_LOADED,NEEDTOKNOW_VERSION));
end

function NeedToKnow_Frame_OnEvent(self, event)
	-- setup needs to be late enough that the icons can find their textures
	if ( event == "PLAYER_LOGIN" ) then
--		NeedToKnow_Settings = NeedToKnow_Settings or NeedToKnow_Defaults;
		if ( not NeedToKnow_Settings ) then
			NeedToKnow_Reset();
		end
		if ( NeedToKnow_Settings["Version"] < NEEDTOKNOW_VERSION ) then
			NeedToKnow_Settings_tmp = NeedToKnow_Defaults;
			for key,value in pairs(NeedToKnow_Settings) do
				if ( NeedToKnow_Defaults[key] ) then
					NeedToKnow_Settings_tmp[key] = value;
				end
			end
			NeedToKnow_Settings_tmp["Version"] = NEEDTOKNOW_VERSION;
			NeedToKnow_Settings = NeedToKnow_Settings_tmp;
		end
		-- thanks to WoWWiki for the settings-transfer code
		NeedToKnow_Initialize();
	end
end

function NeedToKnow_Initialize()
-- called by OnEvent(PLAYER_LOGIN), reset button/command
	local mainFrame = getglobal("NeedToKnow");
	local lockButton = getglobal("NeedToKnow_LockButton");
	local resetButton = getglobal("NeedToKnow_ResetButton");
	local resizeButton = getglobal("NeedToKnow_ResizeButton");
	mainFrame:SetScale(NeedToKnow_Settings["Scale"]);
	NeedToKnow_SetWidth(NeedToKnow_Settings["Width"]);
	if ( NeedToKnow_Settings["Locked"] ) then
		mainFrame:EnableMouse(0);
		lockButton:Hide();
		resetButton:Hide();
		resizeButton:Hide();
	else
		mainFrame:EnableMouse(1);
		lockButton:Show();
		resetButton:Show();
		resizeButton:Show();
	end
	for barNumber = 1,4 do 
		NeedToKnow_Bar_Initialize(barNumber);
	end 
	NeedToKnow_IconContainer_Initialize();
	for iconNumber = 1,7 do 
		NeedToKnow_Icon_Initialize(iconNumber);
	end
end


-- ----------
-- TIMER BARS
-- ----------

function NeedToKnow_Bar_OnLoad()
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	this:RegisterForDrag("LeftButton");
end

function NeedToKnow_Bar_Initialize(barNumber)
-- called by main frame and slash commands
	local thisBar = getglobal("NeedToKnow_Bar"..barNumber);
	local color = NeedToKnow_Settings["BarColor"][barNumber];
	local barEnabled = NeedToKnow_Settings["BarEnabled"][barNumber];
	local barFlash = getglobal("NeedToKnow_Bar"..barNumber.."Flash");
	local barSpark = getglobal("NeedToKnow_Bar"..barNumber.."Spark");
	local barText = getglobal("NeedToKnow_Bar"..barNumber.."Text");
	thisBar.updateTimer = NEEDTOKNOW_UPDATE_INTERVAL_BAR;
	thisBar.startTime = 0;
	thisBar.endTime = GetTime();
	thisBar:SetMinMaxValues(0, 1);
	thisBar:SetValue(1);
	thisBar:SetStatusBarColor(color.r, color.g, color.b);
	thisBar:SetStatusBarTexture("Interface\\AddOns\\NeedToKnow\\Textures\\"..NeedToKnow_Settings["BarTexture"]);
	barSpark:SetPoint("CENTER", thisBar, "RIGHT");
	barFlash:Hide();
	barText:SetText( NeedToKnow_Settings["BuffWatched"][barNumber] or "(none)" );
	if ( barEnabled ) then
		thisBar:SetAlpha(1);
	else
		thisBar:SetAlpha(0.4);
	end
	thisBar:Show();
end

function NeedToKnow_Bar_OnEvent(self, event, localArg1)
	if ( event == "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS" ) then
		local barNumber = self:GetID();
		local barEnabled = NeedToKnow_Settings["BarEnabled"][barNumber];
		if ( barEnabled ) then
			local buffWatched = NeedToKnow_Settings["BuffWatched"][barNumber];
 			if ( string.gsub(localArg1, NEEDTOKNOW_GAIN_MESSAGE_BEFORE.."(.*)"..NEEDTOKNOW_GAIN_MESSAGE_AFTER, "%1") == buffWatched ) then
				self:Show();
			end
		end
	end
end

function NeedToKnow_Bar_OnUpdate(self, elapsed)
	self.updateTimer = self.updateTimer - elapsed;
	if ( self.updateTimer <= 0 ) then 
		self.updateTimer = NEEDTOKNOW_UPDATE_INTERVAL_BAR;
		if ( not NeedToKnow_Settings["Locked"] ) then
			return;
		end
		local buffWatched = NeedToKnow_Settings["BuffWatched"][self:GetID()];
		for buffID = 1,16 do 
			local buffScanned = GetPlayerBuffName(buffID);
			if ( not buffScanned ) then
				break;
			elseif ( buffScanned == buffWatched ) then
				local currentTime = GetTime();
				local timeLeft = GetPlayerBuffTimeLeft(buffID);
				if ( currentTime + timeLeft > self.endTime ) then
					self.startTime = currentTime;
					self.endTime = currentTime + timeLeft;
					self:SetMinMaxValues(0, timeLeft);
				end
				self:SetValue(self.endTime - currentTime);
				local barText = getglobal(self:GetName().."Text");
				barText:SetText( SecondsToTimeAbbrev(timeLeft).." - "..buffWatched );
				local barSpark = getglobal(self:GetName().."Spark");
				local sparkPosition = ((self.endTime - currentTime) / (self.endTime - self.startTime)) * self:GetWidth();
				barSpark:SetPoint("CENTER", self, "LEFT", sparkPosition, 0);
				return;
			end
		end
		self:Hide();
	end
end



-- --------------
-- REMINDER ICONS
-- --------------

function NeedToKnow_IconContainer_OnLoad()
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
end

function NeedToKnow_IconContainer_Initialize()
-- called by main frame and slash commands
	local iconContainer = getglobal("NeedToKnow_IconContainer");
	if ( NeedToKnow_Settings["Locked"] and ( not NeedToKnow_Settings["IconsOn"] or NeedToKnow_Settings["IconsOnlyInCombat"] ) ) then
		iconContainer:Hide();
	else
		iconContainer:Show();
	end
end

function NeedToKnow_IconContainer_OnEvent(self, event)
	if ( NeedToKnow_Settings["IconsOn"] and NeedToKnow_Settings["IconsOnlyInCombat"] and NeedToKnow_Settings["Locked"] ) then
		if ( event == "PLAYER_REGEN_DISABLED" ) then
			self:Show();
		elseif ( event == "PLAYER_REGEN_ENABLED" ) then
			self:Hide();
		end
	end
end

function NeedToKnow_Icon_OnLoad()
	this:RegisterForDrag("LeftButton");
end

function NeedToKnow_Icon_Initialize(iconNumber)
-- called by main frame and slash commands
	local icon = getglobal("NeedToKnow_IconContainer_Icon"..iconNumber);
	local texture = getglobal(icon:GetName().."Texture");
	local highlight = getglobal(icon:GetName().."Highlight");
	local abilityWatched = NeedToKnow_Settings["AbilityWatched"][iconNumber];
	highlight:Hide();
	if ( abilityWatched ) then 
		if ( GetSpellCooldown(abilityWatched) ) then
			texture:SetTexture(GetSpellTexture(abilityWatched));
		elseif ( GetItemCooldown(abilityWatched) ) then
 			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(abilityWatched);
			texture:SetTexture(itemTexture);
		end
	else
		texture:SetTexture(nil);
	end
	if ( not texture:GetTexture() ) then
		texture:SetTexture(0, 0, 0, 0.5);
		if ( NeedToKnow_Settings["Locked"] ) then
			icon:Hide();
		elseif ( not NeedToKnow_Settings["Locked"] ) then
			icon:Show();
		end
	else
		icon:Show();
	end
	texture:Show();
	icon.updateTimer = NEEDTOKNOW_UPDATE_INTERVAL_ICON;
end

function NeedToKnow_Icon_OnUpdate(self, elapsed)
	self.updateTimer = self.updateTimer - elapsed;
	if ( self.updateTimer <= 0 ) then 
		self.updateTimer = NEEDTOKNOW_UPDATE_INTERVAL_ICON;
		if ( not NeedToKnow_Settings["Locked"] ) then
			return;
		end
		local abilityWatched = NeedToKnow_Settings["AbilityWatched"][self:GetID()];
		local texture = getglobal(self:GetName().."Texture");
		local isVisible = texture:IsVisible();
		-- Spell reminders
		local startTime, timeLeft = GetSpellCooldown(abilityWatched);
		if ( timeLeft ) then
			local usable, nomana = IsUsableSpell(abilityWatched);
			if ( isVisible and ( (not usable and not nomana) or timeLeft > 1.5 ) ) then
				texture:Hide(); 
			elseif ( not isVisible and (usable and timeLeft == 0) ) then 
				texture:Show();
			end
			return;
		end
		-- Item reminders
		local startTime, timeLeft = GetItemCooldown(abilityWatched);
		if ( timeLeft ) then
			if ( isVisible and timeLeft > 1.5 ) then
				texture:Hide(); 
			elseif ( not isVisible and timeLeft == 0 ) then 
				texture:Show();
			end
		end
	end
end



-- -----------
-- BAR OPTIONS
-- -----------

function NeedToKnow_Bar_OnEnter(self, motion)
	GameTooltip_SetDefaultAnchor(GameTooltip, this);
	GameTooltip:AddLine(format(NEEDTOKNOW_TOOLTIP,NEEDTOKNOW_VERSION), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, 1);
	if ( NeedToKnow_Settings["Locked"] ) then
		GameTooltip:AddLine(NEEDTOKNOW_INSTRUCTIONS_LOCKED, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
	else
		GameTooltip:AddLine(NEEDTOKNOW_INSTRUCTIONS_UNLOCKED, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
		local barFlash = getglobal(self:GetName().."Flash");
		barFlash:Show();
	end
	GameTooltip:Show();
end

function NeedToKnow_Bar_OnLeave(self, motion)
	GameTooltip:Hide();
	if ( not NeedToKnow_Settings["Locked"] ) then
		local barFlash = getglobal(self:GetName().."Flash");
		barFlash:Hide();
	end
end

function NeedToKnow_Bar_OnMouseDown(self, button)
	if ( button == "RightButton" and not NeedToKnow_Settings["Locked"] ) then
		ToggleDropDownMenu(1, nil, getglobal(self:GetName().."DropDown"), "cursor", 0, 0);
	end
end

function NeedToKnow_Bar_DropDownMenu()

	local info, barEnabled, buffWatched, r, b, g;
	local barNumber = this:GetID();
	-- when menu brought up by right-click, "this" == the bar

	if ( NeedToKnow_Settings ) then
		barEnabled = NeedToKnow_Settings["BarEnabled"][barNumber];
		buffWatched = NeedToKnow_Settings["BuffWatched"][barNumber];
		barColor = NeedToKnow_Settings["BarColor"][barNumber];
		barTexture = NeedToKnow_Settings["BarTexture"];
	else
		barEnabled = nil;
		buffWatched, barTexture = "", "";
		barColor = { r=0, g=0, b=0 };
	end

	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		if ( UIDROPDOWNMENU_MENU_VALUE == NEEDTOKNOW_BAR_SETTEXTURE ) then
			local textureName;
			for i=1, #NEEDTOKNOW_BAR_TEXTURES do
				textureName = NEEDTOKNOW_BAR_TEXTURES[i];
				info = UIDropDownMenu_CreateInfo();
				info.text = textureName;
				info.value = textureName;
				info.func = NeedToKnow_Bar_SetTexture;
				info.arg1 = textureName;
				if ( textureName == barTexture ) then
					info.checked = 1;
				end
				UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
			end
		end
		return;
	end

	-- Menu header
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_BAR_MENU_HEADER..barNumber..": "..buffWatched;
	info.isTitle = 1;
	UIDropDownMenu_AddButton(info);

	-- Set buff
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_BAR_SETBUFF;
	info.tooltipTitle = NEEDTOKNOW_BAR_SETBUFF;
	info.tooltipText = NEEDTOKNOW_BAR_SETBUFF_TOOLTIP;
	info.func = NeedToKnow_Bar_SetBuff_Popup;
	info.arg1 = barNumber;
	UIDropDownMenu_AddButton(info);

	-- Enable bar
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_BAR_ENABLE;
	info.tooltipTitle = NEEDTOKNOW_BAR_ENABLE;
	info.tooltipText = NEEDTOKNOW_BAR_ENABLE_TOOLTIP;
	info.checked = barEnabled;
	info.keepShownOnClick = 1;
	info.func = NeedToKnow_Bar_Toggle;
	info.arg1 = barNumber;
	UIDropDownMenu_AddButton(info);

	-- Set bar color
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_BAR_SETCOLOR;
	info.tooltipTitle = NEEDTOKNOW_BAR_SETCOLOR;
	info.tooltipText = NEEDTOKNOW_BAR_SETCOLOR_TOOLTIP;
	info.hasColorSwatch = 1;
	info.r = barColor.r;
	info.g = barColor.g;
	info.b = barColor.b;
	info.func = UIDropDownMenuButton_OpenColorPicker;
	info.swatchFunc = NeedToKnow_Bar_SetColor;
	info.cancelFunc = NeedToKnow_Bar_CancelColor;
	UIDropDownMenu_AddButton(info);

	-- Set bar texture
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_BAR_SETTEXTURE;
	info.tooltipTitle = NEEDTOKNOW_BAR_SETTEXTURE;
	info.tooltipText = NEEDTOKNOW_BAR_SETTEXTURE_TOOLTIP;
	info.hasArrow = 1;
	UIDropDownMenu_AddButton(info);

end

function NeedToKnow_Bar_SetBuff_Popup(barNumber)
	local dialog = StaticPopup_Show("NEEDTOKNOW_BAR_SETBUFF_POPUP");
	if ( dialog ) then 
		dialog.data = barNumber;
	end
end

StaticPopupDialogs["NEEDTOKNOW_BAR_SETBUFF_POPUP"] = {
	text = NEEDTOKNOW_BAR_SETBUFF_POPUP,
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	maxLetters = 35,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus();
	end,
	OnAccept = function(barNumber)
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText();
		NeedToKnow_Bar_SetBuff(barNumber, text);
	end,
	EditBoxOnEnterPressed = function(barNumber)
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText();
		NeedToKnow_Bar_SetBuff(barNumber, text);
		this:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	OnHide = function()
		if ( ChatFrameEditBox:IsVisible() ) then
			ChatFrameEditBox:SetFocus();
		end
		getglobal(this:GetName().."EditBox"):SetText("");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
};

function NeedToKnow_Bar_SetBuff(barNumber, buffWatched)
	if ( buffWatched ) then
		NeedToKnow_Settings["BuffWatched"][barNumber] = buffWatched;
		NeedToKnow_Bar_Initialize(barNumber);
	end
end

function NeedToKnow_Bar_Toggle(barNumber)
	if ( NeedToKnow_Settings["BarEnabled"][barNumber] ) then
		NeedToKnow_Settings["BarEnabled"][barNumber] = nil;
	else
		NeedToKnow_Settings["BarEnabled"][barNumber] = 1;
	end
	NeedToKnow_Bar_Initialize(barNumber);
end

function NeedToKnow_Bar_SetColor()
	local r,g,b = ColorPickerFrame:GetColorRGB();
	local barNumber = UIDropDownMenu_GetCurrentDropDown():GetParent():GetID();
	NeedToKnow_Settings["BarColor"][barNumber]["r"] = r;
	NeedToKnow_Settings["BarColor"][barNumber]["g"] = g;
	NeedToKnow_Settings["BarColor"][barNumber]["b"] = b;
	NeedToKnow_Bar_Initialize(barNumber);
end;

function NeedToKnow_Bar_CancelColor(previousValues)
	if ( previousValues.r ) then
		local barNumber = UIDropDownMenu_GetCurrentDropDown():GetParent():GetID();
		NeedToKnow_Settings["BarColor"][barNumber]["r"] = previousValues.r;
		NeedToKnow_Settings["BarColor"][barNumber]["g"] = previousValues.g;
		NeedToKnow_Settings["BarColor"][barNumber]["b"] = previousValues.b;
		NeedToKnow_Bar_Initialize(barNumber);
	end
end;

function NeedToKnow_Bar_SetTexture(textureName)
	NeedToKnow_Settings["BarTexture"] = textureName;
	NeedToKnow_Initialize();
end



-- ------------
-- ICON OPTIONS
-- ------------

function NeedToKnow_Icon_OnEnter(self, motion)
	GameTooltip_SetDefaultAnchor(GameTooltip, this);
	GameTooltip:AddLine(format(NEEDTOKNOW_TOOLTIP,NEEDTOKNOW_VERSION), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, 1);
	if ( NeedToKnow_Settings["Locked"] ) then
		GameTooltip:AddLine(NEEDTOKNOW_INSTRUCTIONS_LOCKED, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
	else
		GameTooltip:AddLine(NEEDTOKNOW_INSTRUCTIONS_UNLOCKED, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
		local highlight = getglobal(self:GetName().."Highlight");
		highlight:Show();
	end
	GameTooltip:Show();
end

function NeedToKnow_Icon_OnLeave(self, motion)
	GameTooltip:Hide();
	if ( not NeedToKnow_Settings["Locked"] ) then
		local highlight = getglobal(self:GetName().."Highlight");
		highlight:Hide();
	end
end

function NeedToKnow_Icon_OnMouseDown(self, button)
	if ( button == "RightButton" and not NeedToKnow_Settings["Locked"] ) then
		ToggleDropDownMenu(1, nil, getglobal(self:GetName().."DropDown"), "cursor", 0, 0);
	end
end

function NeedToKnow_Icon_DropDownMenu()

	local info, iconsOn, iconsOnlyInCombat, abilityWatched;
	local iconNumber = this:GetID();
	-- when menu brought up by right-click, "this" == the icon frame
	if ( NeedToKnow_Settings ) then
		iconsOn = NeedToKnow_Settings["IconsOn"];
		iconsOnlyInCombat = NeedToKnow_Settings["IconsOnlyInCombat"];
		abilityWatched = NeedToKnow_Settings["AbilityWatched"][iconNumber];
	else
		iconsOn, iconsOnlyInCombat = nil, nil;
		abilityWatched = "";
	end

	-- Menu header
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_ICON_MENU_HEADER..iconNumber..": "..abilityWatched;
	info.isTitle = 1;
	UIDropDownMenu_AddButton(info);

	-- Set ability watched by icon
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_ICON_SET;
	info.tooltipTitle = NEEDTOKNOW_ICON_SET;
	info.tooltipText = NEEDTOKNOW_ICON_SET_TOOLTIP;
	info.func = NeedToKnow_Icon_Set_Popup;
	info.arg1 = iconNumber;
	UIDropDownMenu_AddButton(info);

	-- Enable icons
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_ICONS_ENABLE;
	info.tooltipTitle = NEEDTOKNOW_ICONS_ENABLE;
	info.tooltipText = NEEDTOKNOW_ICONS_ENABLE_TOOLTIP;
	info.checked = iconsOn;
	info.keepShownOnClick = 1;
	info.func = NeedToKnow_Icons_Toggle;
	UIDropDownMenu_AddButton(info);

	-- Show icons only in combat
	info = UIDropDownMenu_CreateInfo();
	info.text = NEEDTOKNOW_ICONS_ONLYINCOMBAT;
	info.tooltipTitle = NEEDTOKNOW_ICONS_ONLYINCOMBAT;
	info.tooltipText = NEEDTOKNOW_ICONS_ONLYINCOMBAT_TOOLTIP;
	info.checked = iconsOnlyInCombat;
	info.keepShownOnClick = 1;
	info.func = NeedToKnow_Icons_OnlyInCombat;
	UIDropDownMenu_AddButton(info);

end

function NeedToKnow_Icon_Set_Popup(iconNumber)
	local dialog = StaticPopup_Show("NEEDTOKNOW_ICON_SET_POPUP");
	if ( dialog ) then 
		dialog.data = iconNumber;
	end
end

StaticPopupDialogs["NEEDTOKNOW_ICON_SET_POPUP"] = {
	text = NEEDTOKNOW_ICON_SET_POPUP,
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	maxLetters = 35,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus();
	end,
	OnAccept = function(iconNumber)
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText();
		NeedToKnow_Icon_Set(iconNumber, text);
	end,
	EditBoxOnEnterPressed = function(iconNumber)
		local text = getglobal(this:GetParent():GetName().."EditBox"):GetText();
		NeedToKnow_Icon_Set(iconNumber, text);
		this:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	OnHide = function()
		if ( ChatFrameEditBox:IsVisible() ) then
			ChatFrameEditBox:SetFocus();
		end
		getglobal(this:GetName().."EditBox"):SetText("");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
};

function NeedToKnow_Icon_Set(iconNumber, abilityWatched)
	NeedToKnow_Settings["AbilityWatched"][iconNumber] = abilityWatched;
	NeedToKnow_Icon_Initialize(iconNumber);
end

function NeedToKnow_Icons_Toggle()
	if ( NeedToKnow_Settings["IconsOn"] ) then
		NeedToKnow_Settings["IconsOn"] = nil;
	else
		NeedToKnow_Settings["IconsOn"] = 1;
	end
end

function NeedToKnow_Icons_OnlyInCombat()
	if ( NeedToKnow_Settings["IconsOnlyInCombat"] ) then
		NeedToKnow_Settings["IconsOnlyInCombat"] = nil;
	else
		NeedToKnow_Settings["IconsOnlyInCombat"] = 1;
	end
end



-- ----------------------------
-- SLASH COMMANDS & GUI BUTTONS
-- ----------------------------

function NeedToKnow_SlashCommand(cmd)
	if ( cmd == NEEDTOKNOW_CMD_RESET ) then
		NeedToKnow_Reset();
	else 
		NeedToKnow_LockToggle();
	end
end

function NeedToKnow_LockToggle()
	if ( NeedToKnow_Settings["Locked"] ) then
		NeedToKnow_Settings["Locked"] = nil;
	else
		NeedToKnow_Settings["Locked"] = 1;
	end
	NeedToKnow_Initialize();
end


function NeedToKnow_Reset()
	NeedToKnow_Settings = {};
	NeedToKnow_CopyTable(NeedToKnow_Defaults, NeedToKnow_Settings);
	NeedToKnow:ClearAllPoints();
	NeedToKnow:SetPoint("TOPLEFT", "UIParent", "CENTER", 0, 0);
	NeedToKnow_Initialize();
end

function NeedToKnow_CopyTable(source, destination)
	for key, value in pairs(source) do
		if (type(value) == "table") then
			destination[key] = {};
			NeedToKnow_CopyTable(value, destination[key]);
		else
			destination[key] = value;
		end
	end
end
-- Thanks to eCastingBar for the CopyTable code

function NeedToKnow_GUIButton_OnEnter(shortText, longText)
	local tooltip = getglobal("GameTooltip");
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(tooltip, this);
		tooltip:AddLine(shortText, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, 1);
		tooltip:AddLine(longText, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
		tooltip:Show();
	else
		tooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT");
		tooltip:SetText(shortText);
	end
end

function NeedToKnow_StartSizing(self, button)
	self:SetScript("OnUpdate", NeedToKnow_SizeUpdate)
	self.oldCursorX, self.oldCursorY = GetCursorPosition(UIParent);
	local mainFrame = self:GetParent();
	mainFrame.oldScale = mainFrame:GetScale();
	mainFrame.oldHeight = mainFrame:GetHeight();
	mainFrame.oldWidth = mainFrame:GetWidth();
	mainFrame.oldX = mainFrame:GetLeft();
	mainFrame.oldY = mainFrame:GetTop();
end

function NeedToKnow_SizeUpdate()
	local mainFrame = this:GetParent();
	local uIParentScale = UIParent:GetScale();
	this.newCursorX, this.newCursorY = GetCursorPosition(UIParent);
	mainFrame.newScale = max(0.3, mainFrame.oldScale + (this.oldCursorY - this.newCursorY)/(mainFrame.oldHeight * uIParentScale));
	mainFrame:SetScale(mainFrame.newScale);
	mainFrame.newX = mainFrame.oldX * mainFrame.oldScale / mainFrame.newScale;
	mainFrame.newY = mainFrame.oldY * mainFrame.oldScale / mainFrame.newScale;
	mainFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", mainFrame.newX, mainFrame.newY );
	mainFrame.newWidth = max(50, ((this.newCursorX - this.oldCursorX)/uIParentScale + mainFrame.oldWidth * mainFrame.oldScale)/mainFrame.newScale);
	NeedToKnow_SetWidth(mainFrame.newWidth);
end

function NeedToKnow_SetWidth(width)
	local mainFrame = getglobal("NeedToKnow");
	mainFrame:SetWidth(width);
	for barNumber = 1,4 do 
		local bar = getglobal(mainFrame:GetName().."_Bar"..barNumber);
		local barBorder = getglobal(bar:GetName().."Border");
		local barFlash = getglobal(bar:GetName().."Flash");
		local widthMultiplier = width/NEEDTOKNOW_DEFAULT_FRAME_WIDTH;
		bar:SetWidth( widthMultiplier * NEEDTOKNOW_DEFAULT_BAR_WIDTH );
		barBorder:SetWidth( widthMultiplier * NEEDTOKNOW_DEFAULT_BAR_BORDER_WIDTH );
		barFlash:SetWidth( widthMultiplier * NEEDTOKNOW_DEFAULT_BAR_BORDER_WIDTH );
	end
end

function NeedToKnow_StopSizing(self, button)
	self:SetScript("OnUpdate", nil)
	local mainFrame = self:GetParent();
	NeedToKnow_Settings["Scale"] = mainFrame:GetScale();
	NeedToKnow_Settings["Width"] = mainFrame:GetWidth();
end






